import java.util.*;

public class Dice
{
	private final int NUM_FACES = 5;
	
	public int getNumFaces()
	{
		return NUM_FACES;
	}
	
	private int[] faces = new int[NUM_FACES];
	
	public int[] getKeptFaces()
	{
		int[] keptCopy = new int[numKept];
		int nKept = 0;
		
		for( int i=0; i<NUM_FACES; i++ )
		{
			if( isKept[i] ) 
			{
				keptCopy[ nKept++ ] = faces[i];
			}
		}
		
		Arrays.sort( keptCopy );		
		return keptCopy;
	}
	
	public int[] getStillRollingFaces()
	{
		int[] stillRollingCopy = new int[numStillRolling];
		int nStillRolling = 0;
		
		for( int i=0; i<NUM_FACES; i++ )
		{
			if( !isKept[i] ) 
			{
				stillRollingCopy[ nStillRolling++ ] = faces[i];
			}
		}
		
		Arrays.sort( stillRollingCopy );		
		return stillRollingCopy;

	}
	
	public void setFaces( int[] faces )
	{
		if( faces.length == NUM_FACES )
			this.faces = faces;
	}	
	private boolean[] isKept = new boolean[NUM_FACES];
	private int numKept = 0;
	public int getNumKept()
	{
		return numKept;
	}
	
	private int numStillRolling = NUM_FACES;

	public int getNumStillRolling()
	{
		return numStillRolling;
	}
	private String type = "Invalid";
	private int score = 0;
	
	public Dice()
	{
	}


	public static void main(String[] args)
	{

		int[] roll;
		// Set Full House		
		roll = new int[]{3,3,2,3,2};
		Dice d = new Dice();
		d.setFaces( roll );	
		
		System.out.println( "\nInitial" );
		System.out.println( "Kept: " + Arrays.toString( d.getKeptFaces() ) );
		System.out.println( "Still Rolling: " + Arrays.toString( d.getStillRollingFaces() ) );
		
		d.keep( 2 );
		d.keep( 2 );
		
		System.out.println( "\nMarked 2's" );
		System.out.println( "Kept: " + Arrays.toString( d.getKeptFaces() ) );
		System.out.println( "Still Rolling: " + Arrays.toString( d.getStillRollingFaces() ) );
		
		
	}
	
	public void keep( int face )
	{
		int i = findFace( face, false );
		
//		System.out.println( i );
		
		if( i != -1 )
		{
			isKept[i] = true;
			numKept++;
			numStillRolling--;
		}
	}

	public void keepAll()
	{
		for( int i=0; i<NUM_FACES; i++ )
			isKept[i] = true;
		numKept = NUM_FACES;
	}

	public void putBack( int face )
	{
		int i = findFace( face, true );
		
		if( i != -1 )
		{
			isKept[i] = false;
			numKept--;
			numStillRolling++;
		}
	}
	
	private int findFace( int face, boolean kept )
	{
		for( int i=0; i<NUM_FACES; i++ )
		{
//			System.out.println( face + ", " + faces[i] + ", " + !isKept[i] );
			if( (face == faces[i]) && isKept[i]==kept )
				return i;
		}
		return -1;
	}


	private final int NUM_DIE_FACES = 6;
	
	public void roll()
	{
		for( int i=0; i<NUM_FACES; i++ )
		{
			if( !isKept[i] ) 
			{
				faces[i] = (int)(Math.random()*6) + 1;
			}
		}
	}

	public int[] counts()
	{
		int[] counts = new int[NUM_DIE_FACES];
		
		for( int i=0; i<NUM_FACES; i++ )
			counts[ faces[i]-1 ]++;
			
		return counts;
	}
		
	public int[] getFaces()
	{
		
		int[] facesCopy = new int[ NUM_FACES ];
		
		System.arraycopy( faces, 0, facesCopy, 0, NUM_FACES );

		Arrays.sort( facesCopy );		

		return facesCopy;
	}		
	
	public void setScore( String type, int score )
	{
		this.type = type;
		this.score = score;
	}
	
	public int getFacesTotal()
	{
		int sum = 0;
		
		for( int face : faces ) sum += face;
		
		return sum;
	}
	
	public int getScore()
	{
		return score;
	}
	public String getType()
	{
		return type;
	}
	
	public String toString()
	{
		String s = "Faces " + Arrays.toString( getFaces() )+ " has been scored as a " + type + " with value " + score;
		
		return s;
	}
}